import { DispatcherClient as PbDispatcherClient, AssignedAction } from '../../protoc/dispatcher';
import { ClientConfig } from '../hatchet-client/client-config';
import { Logger } from '../../util/logger';
import { z } from 'zod';
import { DispatcherClient } from './dispatcher-client';
import { Heartbeat } from './heartbeat/heartbeat-controller';
declare enum ListenStrategy {
    LISTEN_STRATEGY_V1 = 1,
    LISTEN_STRATEGY_V2 = 2
}
export declare const ActionObject: z.ZodObject<{
    tenantId: z.ZodString;
    jobId: z.ZodString;
    jobName: z.ZodString;
    jobRunId: z.ZodString;
    stepId: z.ZodString;
    stepRunId: z.ZodString;
    actionId: z.ZodString;
    actionType: z.ZodEffects<z.ZodOptional<z.ZodNumber>, number | undefined, unknown>;
    actionPayload: z.ZodString;
    workflowRunId: z.ZodString;
    getGroupKeyRunId: z.ZodOptional<z.ZodString>;
    stepName: z.ZodString;
}, "strip", z.ZodTypeAny, {
    tenantId: string;
    stepRunId: string;
    workflowRunId: string;
    jobId: string;
    jobName: string;
    jobRunId: string;
    stepId: string;
    actionId: string;
    actionPayload: string;
    stepName: string;
    getGroupKeyRunId?: string | undefined;
    actionType?: number | undefined;
}, {
    tenantId: string;
    stepRunId: string;
    workflowRunId: string;
    jobId: string;
    jobName: string;
    jobRunId: string;
    stepId: string;
    actionId: string;
    actionPayload: string;
    stepName: string;
    getGroupKeyRunId?: string | undefined;
    actionType?: unknown;
}>;
export type Action = z.infer<typeof ActionObject>;
export declare class ActionListener {
    config: ClientConfig;
    client: PbDispatcherClient;
    workerId: string;
    logger: Logger;
    lastConnectionAttempt: number;
    retries: number;
    retryInterval: number;
    retryCount: number;
    done: boolean;
    listenStrategy: ListenStrategy;
    heartbeat: Heartbeat;
    constructor(client: DispatcherClient, workerId: string, retryInterval?: number, retryCount?: number);
    actions: () => AsyncGenerator<{
        tenantId: string;
        stepRunId: string;
        workflowRunId: string;
        jobId: string;
        jobName: string;
        jobRunId: string;
        stepId: string;
        actionId: string;
        actionPayload: string;
        stepName: string;
        getGroupKeyRunId?: string | undefined;
        actionType?: number | undefined;
    }, void, unknown>;
    setListenStrategy(strategy: ListenStrategy): Promise<void>;
    getListenStrategy(): Promise<ListenStrategy>;
    incrementRetries(): Promise<void>;
    getListenClient(): Promise<AsyncIterable<AssignedAction>>;
    unregister(): Promise<import("../../protoc/dispatcher").WorkerUnsubscribeResponse>;
}
export {};
