"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return", awaitReturn), i[Symbol.asyncIterator] = function () { return this; }, i;
    function awaitReturn(f) { return function (v) { return Promise.resolve(v).then(f, reject); }; }
    function verb(n, f) { if (g[n]) { i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; if (f) i[n] = f(i[n]); } }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListenerClient = exports.PollingAsyncListener = exports.RunEventType = void 0;
// eslint-disable-next-line max-classes-per-file
const nice_grpc_1 = require("nice-grpc");
const events_1 = require("events");
const dispatcher_1 = require("../../protoc/dispatcher");
const hatchet_error_1 = __importDefault(require("../../util/errors/hatchet-error"));
const logger_1 = require("../../util/logger");
const sleep_1 = __importDefault(require("../../util/sleep"));
const data_contracts_1 = require("../rest/generated/data-contracts");
const child_listener_client_1 = require("./child-listener-client");
const DEFAULT_EVENT_LISTENER_RETRY_INTERVAL = 5; // seconds
const DEFAULT_EVENT_LISTENER_RETRY_COUNT = 5;
const DEFAULT_EVENT_LISTENER_POLL_INTERVAL = 5000; // milliseconds
// eslint-disable-next-line no-shadow
var RunEventType;
(function (RunEventType) {
    RunEventType["STEP_RUN_EVENT_TYPE_STARTED"] = "STEP_RUN_EVENT_TYPE_STARTED";
    RunEventType["STEP_RUN_EVENT_TYPE_COMPLETED"] = "STEP_RUN_EVENT_TYPE_COMPLETED";
    RunEventType["STEP_RUN_EVENT_TYPE_FAILED"] = "STEP_RUN_EVENT_TYPE_FAILED";
    RunEventType["STEP_RUN_EVENT_TYPE_CANCELLED"] = "STEP_RUN_EVENT_TYPE_CANCELLED";
    RunEventType["STEP_RUN_EVENT_TYPE_TIMED_OUT"] = "STEP_RUN_EVENT_TYPE_TIMED_OUT";
    RunEventType["STEP_RUN_EVENT_TYPE_STREAM"] = "STEP_RUN_EVENT_TYPE_STREAM";
    RunEventType["WORKFLOW_RUN_EVENT_TYPE_STARTED"] = "WORKFLOW_RUN_EVENT_TYPE_STARTED";
    RunEventType["WORKFLOW_RUN_EVENT_TYPE_COMPLETED"] = "WORKFLOW_RUN_EVENT_TYPE_COMPLETED";
    RunEventType["WORKFLOW_RUN_EVENT_TYPE_FAILED"] = "WORKFLOW_RUN_EVENT_TYPE_FAILED";
    RunEventType["WORKFLOW_RUN_EVENT_TYPE_CANCELLED"] = "WORKFLOW_RUN_EVENT_TYPE_CANCELLED";
    RunEventType["WORKFLOW_RUN_EVENT_TYPE_TIMED_OUT"] = "WORKFLOW_RUN_EVENT_TYPE_TIMED_OUT";
})(RunEventType || (exports.RunEventType = RunEventType = {}));
const stepEventTypeMap = {
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_STARTED]: RunEventType.STEP_RUN_EVENT_TYPE_STARTED,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_COMPLETED]: RunEventType.STEP_RUN_EVENT_TYPE_COMPLETED,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_FAILED]: RunEventType.STEP_RUN_EVENT_TYPE_FAILED,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_CANCELLED]: RunEventType.STEP_RUN_EVENT_TYPE_CANCELLED,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_TIMED_OUT]: RunEventType.STEP_RUN_EVENT_TYPE_TIMED_OUT,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_STREAM]: RunEventType.STEP_RUN_EVENT_TYPE_STREAM,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_UNKNOWN]: undefined,
    [dispatcher_1.ResourceEventType.UNRECOGNIZED]: undefined,
};
const workflowEventTypeMap = {
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_STARTED]: RunEventType.WORKFLOW_RUN_EVENT_TYPE_STARTED,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_COMPLETED]: RunEventType.WORKFLOW_RUN_EVENT_TYPE_COMPLETED,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_FAILED]: RunEventType.WORKFLOW_RUN_EVENT_TYPE_FAILED,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_CANCELLED]: RunEventType.WORKFLOW_RUN_EVENT_TYPE_CANCELLED,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_TIMED_OUT]: RunEventType.WORKFLOW_RUN_EVENT_TYPE_TIMED_OUT,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_UNKNOWN]: undefined,
    [dispatcher_1.ResourceEventType.RESOURCE_EVENT_TYPE_STREAM]: undefined,
    [dispatcher_1.ResourceEventType.UNRECOGNIZED]: undefined,
};
const resourceTypeMap = {
    [dispatcher_1.ResourceType.RESOURCE_TYPE_STEP_RUN]: stepEventTypeMap,
    [dispatcher_1.ResourceType.RESOURCE_TYPE_WORKFLOW_RUN]: workflowEventTypeMap,
    [dispatcher_1.ResourceType.RESOURCE_TYPE_UNKNOWN]: undefined,
    [dispatcher_1.ResourceType.UNRECOGNIZED]: undefined,
};
const workflowStatusMap = {
    [data_contracts_1.WorkflowRunStatus.SUCCEEDED]: RunEventType.WORKFLOW_RUN_EVENT_TYPE_COMPLETED,
    [data_contracts_1.WorkflowRunStatus.FAILED]: RunEventType.WORKFLOW_RUN_EVENT_TYPE_FAILED,
    [data_contracts_1.WorkflowRunStatus.CANCELLED]: RunEventType.WORKFLOW_RUN_EVENT_TYPE_CANCELLED,
    [data_contracts_1.WorkflowRunStatus.PENDING]: undefined,
    [data_contracts_1.WorkflowRunStatus.RUNNING]: undefined,
    [data_contracts_1.WorkflowRunStatus.QUEUED]: undefined,
};
class PollingAsyncListener {
    constructor(workflowRunid, client) {
        this.q = [];
        this.eventEmitter = new events_1.EventEmitter();
        this.client = client;
        this.listen(workflowRunid);
        this.polling(workflowRunid);
    }
    emit(event) {
        this.q.push(event);
        this.eventEmitter.emit('event');
    }
    listen(workflowRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a, e_1, _b, _c;
            var _d;
            let listener = this.client.client.subscribeToWorkflowEvents({
                workflowRunId,
            });
            const res = yield this.getWorkflowRun(workflowRunId);
            if (res) {
                this.emit(res);
                this.close();
            }
            try {
                try {
                    for (var _e = true, listener_1 = __asyncValues(listener), listener_1_1; listener_1_1 = yield listener_1.next(), _a = listener_1_1.done, !_a; _e = true) {
                        _c = listener_1_1.value;
                        _e = false;
                        const workflowEvent = _c;
                        const eventType = (_d = resourceTypeMap[workflowEvent.resourceType]) === null || _d === void 0 ? void 0 : _d[workflowEvent.eventType];
                        if (eventType) {
                            if (eventType === RunEventType.WORKFLOW_RUN_EVENT_TYPE_COMPLETED) {
                                // OPTIMZATION - consider including the workflow run data in the event?
                                const data = yield this.getWorkflowRun(workflowRunId);
                                if (data) {
                                    this.emit(data);
                                }
                            }
                            else {
                                this.emit({
                                    type: eventType,
                                    payload: workflowEvent.eventPayload,
                                });
                            }
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (!_e && !_a && (_b = listener_1.return)) yield _b.call(listener_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            catch (e) {
                if (e.code === nice_grpc_1.Status.CANCELLED) {
                    return;
                }
                if (e.code === nice_grpc_1.Status.UNAVAILABLE) {
                    listener = yield this.retrySubscribe(workflowRunId);
                }
            }
            setTimeout(() => this.close(), DEFAULT_EVENT_LISTENER_POLL_INTERVAL * 5);
        });
    }
    retrySubscribe(workflowRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            let retries = 0;
            while (retries < DEFAULT_EVENT_LISTENER_RETRY_COUNT) {
                try {
                    yield (0, sleep_1.default)(DEFAULT_EVENT_LISTENER_RETRY_INTERVAL);
                    const listener = this.client.client.subscribeToWorkflowEvents({
                        workflowRunId,
                    });
                    return listener;
                }
                catch (e) {
                    retries += 1;
                }
            }
            throw new hatchet_error_1.default(`Could not subscribe to the worker after ${DEFAULT_EVENT_LISTENER_RETRY_COUNT} retries`);
        });
    }
    getWorkflowRun(workflowRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c;
            try {
                const res = yield this.client.api.workflowRunGet(this.client.config.tenant_id, workflowRunId);
                const stepRuns = (_c = (_b = (_a = res.data.jobRuns) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.stepRuns) !== null && _c !== void 0 ? _c : [];
                const stepRunOutput = stepRuns.reduce((acc, stepRun) => {
                    var _a;
                    acc[((_a = stepRun.step) === null || _a === void 0 ? void 0 : _a.readableId) || ''] = JSON.parse(stepRun.output || '{}');
                    return acc;
                }, {});
                if (Object.keys(workflowStatusMap).includes(res.data.status)) {
                    const type = workflowStatusMap[res.data.status];
                    if (!type)
                        return undefined;
                    return {
                        type,
                        payload: JSON.stringify(stepRunOutput),
                    };
                }
                return undefined;
            }
            catch (e) {
                throw new hatchet_error_1.default(e.message);
            }
        });
    }
    polling(workflowRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.pollInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const result = yield this.getWorkflowRun(workflowRunId);
                    if (result) {
                        this.emit(result);
                        this.close();
                    }
                }
                catch (e) {
                    // TODO error handling
                }
            }), DEFAULT_EVENT_LISTENER_POLL_INTERVAL);
        });
    }
    close() {
        if (this.pollInterval) {
            clearInterval(this.pollInterval);
        }
    }
    stream() {
        return __asyncGenerator(this, arguments, function* stream_1() {
            var _a, e_2, _b, _c;
            try {
                for (var _d = true, _e = __asyncValues((0, events_1.on)(this.eventEmitter, 'event')), _f; _f = yield __await(_e.next()), _a = _f.done, !_a; _d = true) {
                    _c = _f.value;
                    _d = false;
                    const _ = _c;
                    while (this.q.length > 0) {
                        const r = this.q.shift();
                        if (r) {
                            yield yield __await(r);
                        }
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) yield __await(_b.call(_e));
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
    }
}
exports.PollingAsyncListener = PollingAsyncListener;
class ListenerClient {
    constructor(config, channel, factory, api) {
        this.childListeners = {};
        this.config = config;
        this.client = factory.create(dispatcher_1.DispatcherDefinition, channel);
        this.logger = new logger_1.Logger(`Listener`, config.log_level);
        this.api = api;
    }
    getChildListener(workflowRunId, parentWorkflowRunId) {
        if (!this.childListeners[parentWorkflowRunId]) {
            this.childListeners[parentWorkflowRunId] = new child_listener_client_1.GrpcPooledListener(this, () => {
                // cleanup listener when all children are done
                delete this.childListeners[parentWorkflowRunId];
            });
        }
        return this.childListeners[parentWorkflowRunId].subscribe({
            workflowRunId,
        });
    }
    get(workflowRunId) {
        const listener = new PollingAsyncListener(workflowRunId, this);
        return listener;
    }
    stream(workflowRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.get(workflowRunId).stream();
        });
    }
}
exports.ListenerClient = ListenerClient;
