import { HatchetClient } from '../hatchet-client';
import { Action, ActionListener } from '../dispatcher/action-listener';
import { StepActionEvent, StepActionEventType, GroupKeyActionEvent, GroupKeyActionEventType } from '../../protoc/dispatcher';
import HatchetPromise from '../../util/hatchet-promise/hatchet-promise';
import { Workflow } from '../../workflow';
import { Logger } from '../../util/logger';
import { WebhookHandler } from './handler';
import { WebhookWorkerCreateRequest } from '../rest/generated/data-contracts';
import { Context, StepRunFunction } from '../../step';
export type ActionRegistry = Record<Action['actionId'], Function>;
export declare class Worker {
    client: HatchetClient;
    name: string;
    killing: boolean;
    handle_kill: boolean;
    action_registry: ActionRegistry;
    listener: ActionListener | undefined;
    futures: Record<Action['stepRunId'], HatchetPromise<any>>;
    contexts: Record<Action['stepRunId'], Context<any, any>>;
    maxRuns?: number;
    logger: Logger;
    registeredWorkflowPromises: Array<Promise<any>>;
    registeredWorkflowIds: string[];
    constructor(client: HatchetClient, options: {
        name: string;
        handleKill?: boolean;
        maxRuns?: number;
    });
    private registerActions;
    getHandler(workflow: Workflow): WebhookHandler;
    registerWebhook(webhook: WebhookWorkerCreateRequest): Promise<import("axios").AxiosResponse<import("../rest/generated/data-contracts").WebhookWorker, any>>;
    registerWorkflow(initWorkflow: Workflow): Promise<void>;
    register_workflow(initWorkflow: Workflow): Promise<void>;
    registerAction<T, K>(actionId: string, action: StepRunFunction<T, K>): void;
    handleStartStepRun(action: Action): Promise<void>;
    handleStartGroupKeyRun(action: Action): Promise<void>;
    getStepActionEvent(action: Action, eventType: StepActionEventType, payload?: any): StepActionEvent;
    getGroupKeyActionEvent(action: Action, eventType: GroupKeyActionEventType, payload?: any): GroupKeyActionEvent;
    handleCancelStepRun(action: Action): Promise<void>;
    stop(): Promise<void>;
    exitGracefully(handleKill: boolean): Promise<void>;
    start(): Promise<void>;
    handleAction(action: Action): Promise<void>;
}
