"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Worker = void 0;
const hatchet_error_1 = __importDefault(require("../../util/errors/hatchet-error"));
const dispatcher_1 = require("../../protoc/dispatcher");
const hatchet_promise_1 = __importDefault(require("../../util/hatchet-promise/hatchet-promise"));
const workflows_1 = require("../../protoc/workflows");
const logger_1 = require("../../util/logger");
const handler_1 = require("./handler");
const step_1 = require("../../step");
class Worker {
    constructor(client, options) {
        this.futures = {};
        this.contexts = {};
        this.registeredWorkflowPromises = [];
        this.registeredWorkflowIds = [];
        this.client = client;
        this.name = this.client.config.namespace + options.name;
        this.action_registry = {};
        this.maxRuns = options.maxRuns;
        process.on('SIGTERM', () => this.exitGracefully(true));
        process.on('SIGINT', () => this.exitGracefully(true));
        this.killing = false;
        this.handle_kill = options.handleKill === undefined ? true : options.handleKill;
        this.logger = new logger_1.Logger(`Worker/${this.name}`, this.client.config.log_level);
    }
    registerActions(workflow) {
        var _a;
        const newActions = workflow.steps.reduce((acc, step) => {
            acc[`${workflow.id}:${step.name}`] = step.run;
            return acc;
        }, {});
        const onFailureAction = workflow.onFailure
            ? {
                [`${workflow.id}-on-failure:${workflow.onFailure.name}`]: workflow.onFailure.run,
            }
            : {};
        this.action_registry = Object.assign(Object.assign(Object.assign({}, this.action_registry), newActions), onFailureAction);
        this.action_registry = ((_a = workflow.concurrency) === null || _a === void 0 ? void 0 : _a.name)
            ? Object.assign(Object.assign({}, this.action_registry), { [`${workflow.id}:${workflow.concurrency.name}`]: workflow.concurrency.key }) : Object.assign({}, this.action_registry);
    }
    getHandler(workflow) {
        const wf = Object.assign(Object.assign({}, workflow), { id: this.client.config.namespace + workflow.id });
        this.registerActions(wf);
        return new handler_1.WebhookHandler(this);
    }
    registerWebhook(webhook) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.client.admin.webhook_create(Object.assign(Object.assign({}, webhook), { workflows: this.registeredWorkflowIds }));
        });
    }
    // @deprecated
    registerWorkflow(initWorkflow) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.register_workflow(initWorkflow);
        });
    }
    register_workflow(initWorkflow) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            const workflow = Object.assign(Object.assign({}, initWorkflow), { id: this.client.config.namespace + initWorkflow.id });
            try {
                const concurrency = ((_a = workflow.concurrency) === null || _a === void 0 ? void 0 : _a.name)
                    ? {
                        action: `${workflow.id}:${workflow.concurrency.name}`,
                        maxRuns: workflow.concurrency.maxRuns || 1,
                        limitStrategy: workflow.concurrency.limitStrategy || workflows_1.ConcurrencyLimitStrategy.CANCEL_IN_PROGRESS,
                    }
                    : undefined;
                const onFailureJob = workflow.onFailure
                    ? {
                        name: `${workflow.id}-on-failure`,
                        description: workflow.description,
                        steps: [
                            {
                                readableId: workflow.onFailure.name,
                                action: `${workflow.id}-on-failure:${workflow.onFailure.name}`,
                                timeout: workflow.onFailure.timeout || '60s',
                                inputs: '{}',
                                parents: [],
                                userData: '{}',
                                retries: workflow.onFailure.retries || 0,
                                rateLimits: (_b = workflow.onFailure.rate_limits) !== null && _b !== void 0 ? _b : [],
                            },
                        ],
                    }
                    : undefined;
                this.registeredWorkflowIds.push(workflow.id);
                const registeredWorkflow = this.client.admin.put_workflow({
                    name: workflow.id,
                    description: workflow.description,
                    version: workflow.version || '',
                    eventTriggers: workflow.on.event ? [this.client.config.namespace + workflow.on.event] : [],
                    cronTriggers: workflow.on.cron ? [workflow.on.cron] : [],
                    scheduledTriggers: [],
                    concurrency,
                    scheduleTimeout: workflow.scheduleTimeout,
                    onFailureJob,
                    jobs: [
                        {
                            name: workflow.id,
                            description: workflow.description,
                            steps: workflow.steps.map((step) => {
                                var _a, _b;
                                return ({
                                    readableId: step.name,
                                    action: `${workflow.id}:${step.name}`,
                                    timeout: step.timeout || '60s',
                                    inputs: '{}',
                                    parents: (_a = step.parents) !== null && _a !== void 0 ? _a : [],
                                    userData: '{}',
                                    retries: step.retries || 0,
                                    rateLimits: (_b = step.rate_limits) !== null && _b !== void 0 ? _b : [], // Add the missing rateLimits property
                                });
                            }),
                        },
                    ],
                });
                this.registeredWorkflowPromises.push(registeredWorkflow);
                yield registeredWorkflow;
            }
            catch (e) {
                throw new hatchet_error_1.default(`Could not register workflow: ${e.message}`);
            }
            this.registerActions(workflow);
        });
    }
    registerAction(actionId, action) {
        this.action_registry[actionId] = action;
    }
    handleStartStepRun(action) {
        return __awaiter(this, void 0, void 0, function* () {
            const { actionId } = action;
            try {
                const context = new step_1.Context(action, this.client);
                this.contexts[action.stepRunId] = context;
                const step = this.action_registry[actionId];
                if (!step) {
                    this.logger.error(`Could not find step '${actionId}'`);
                    return;
                }
                const run = () => __awaiter(this, void 0, void 0, function* () {
                    return step(context);
                });
                const success = (result) => {
                    this.logger.info(`Step run ${action.stepRunId} succeeded`);
                    try {
                        // Send the action event to the dispatcher
                        const event = this.getStepActionEvent(action, dispatcher_1.StepActionEventType.STEP_EVENT_TYPE_COMPLETED, result);
                        this.client.dispatcher.sendStepActionEvent(event).catch((e) => {
                            this.logger.error(`Could not send action event: ${e.message}`);
                        });
                        // delete the run from the futures
                        delete this.futures[action.stepRunId];
                    }
                    catch (e) {
                        this.logger.error(`Could not send action event: ${e.message}`);
                    }
                };
                const failure = (error) => {
                    this.logger.error(`Step run ${action.stepRunId} failed: ${error.message}`);
                    if (error.stack) {
                        this.logger.error(error.stack);
                    }
                    try {
                        // Send the action event to the dispatcher
                        const event = this.getStepActionEvent(action, dispatcher_1.StepActionEventType.STEP_EVENT_TYPE_FAILED, {
                            message: error === null || error === void 0 ? void 0 : error.message,
                            stack: error === null || error === void 0 ? void 0 : error.stack,
                        });
                        this.client.dispatcher.sendStepActionEvent(event).catch((e) => {
                            this.logger.error(`Could not send action event: ${e.message}`);
                        });
                        // delete the run from the futures
                        delete this.futures[action.stepRunId];
                    }
                    catch (e) {
                        this.logger.error(`Could not send action event: ${e.message}`);
                    }
                };
                const future = new hatchet_promise_1.default(run().then(success).catch(failure));
                this.futures[action.stepRunId] = future;
                // Send the action event to the dispatcher
                const event = this.getStepActionEvent(action, dispatcher_1.StepActionEventType.STEP_EVENT_TYPE_STARTED);
                this.client.dispatcher.sendStepActionEvent(event).catch((e) => {
                    this.logger.error(`Could not send action event: ${e.message}`);
                });
                yield future.promise;
            }
            catch (e) {
                this.logger.error(`Could not send action event: ${e.message}`);
            }
        });
    }
    handleStartGroupKeyRun(action) {
        return __awaiter(this, void 0, void 0, function* () {
            const { actionId } = action;
            try {
                const context = new step_1.Context(action, this.client);
                const key = action.getGroupKeyRunId;
                if (!key) {
                    this.logger.error(`No group key run id provided for action ${actionId}`);
                    return;
                }
                this.contexts[key] = context;
                this.logger.debug(`Starting group key run ${key}`);
                const step = this.action_registry[actionId];
                if (!step) {
                    this.logger.error(`Could not find step '${actionId}'`);
                    return;
                }
                const run = () => __awaiter(this, void 0, void 0, function* () {
                    return step(context);
                });
                const success = (result) => {
                    this.logger.info(`Step run ${action.stepRunId} succeeded`);
                    try {
                        // Send the action event to the dispatcher
                        const event = this.getGroupKeyActionEvent(action, dispatcher_1.GroupKeyActionEventType.GROUP_KEY_EVENT_TYPE_COMPLETED, result);
                        this.client.dispatcher.sendGroupKeyActionEvent(event).catch((e) => {
                            this.logger.error(`Could not send action event: ${e.message}`);
                        });
                        // delete the run from the futures
                        delete this.futures[key];
                    }
                    catch (e) {
                        this.logger.error(`Could not send action event: ${e.message}`);
                    }
                };
                const failure = (error) => {
                    this.logger.error(`Step run ${key} failed: ${error.message}`);
                    try {
                        // Send the action event to the dispatcher
                        const event = this.getGroupKeyActionEvent(action, dispatcher_1.GroupKeyActionEventType.GROUP_KEY_EVENT_TYPE_FAILED, error);
                        this.client.dispatcher.sendGroupKeyActionEvent(event).catch((e) => {
                            this.logger.error(`Could not send action event: ${e.message}`);
                        });
                        // delete the run from the futures
                        delete this.futures[key];
                    }
                    catch (e) {
                        this.logger.error(`Could not send action event: ${e.message}`);
                    }
                };
                const future = new hatchet_promise_1.default(run().then(success).catch(failure));
                this.futures[key] = future;
                // Send the action event to the dispatcher
                const event = this.getGroupKeyActionEvent(action, dispatcher_1.GroupKeyActionEventType.GROUP_KEY_EVENT_TYPE_STARTED);
                this.client.dispatcher.sendGroupKeyActionEvent(event).catch((e) => {
                    this.logger.error(`Could not send action event: ${e.message}`);
                });
                yield future.promise;
            }
            catch (e) {
                this.logger.error(`Could not send action event: ${e.message}`);
            }
        });
    }
    getStepActionEvent(action, eventType, payload = '') {
        return {
            workerId: this.name,
            jobId: action.jobId,
            jobRunId: action.jobRunId,
            stepId: action.stepId,
            stepRunId: action.stepRunId,
            actionId: action.actionId,
            eventTimestamp: new Date(),
            eventType,
            eventPayload: JSON.stringify(payload),
        };
    }
    getGroupKeyActionEvent(action, eventType, payload = '') {
        if (!action.getGroupKeyRunId) {
            throw new hatchet_error_1.default('No group key run id provided');
        }
        return {
            workerId: this.name,
            workflowRunId: action.workflowRunId,
            getGroupKeyRunId: action.getGroupKeyRunId,
            actionId: action.actionId,
            eventTimestamp: new Date(),
            eventType,
            eventPayload: JSON.stringify(payload),
        };
    }
    handleCancelStepRun(action) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.logger.info(`Cancelling step run ${action.stepRunId}`);
                const { stepRunId } = action;
                const future = this.futures[stepRunId];
                const context = this.contexts[stepRunId];
                if (context && context.controller) {
                    context.controller.abort('Cancelled by worker');
                    delete this.contexts[stepRunId];
                }
                if (future) {
                    future.promise.catch(() => {
                        this.logger.info(`Cancelled step run ${action.stepRunId}`);
                    });
                    future.cancel('Cancelled by worker');
                    yield future.promise;
                    delete this.futures[stepRunId];
                }
            }
            catch (e) {
                this.logger.error(`Could not cancel step run: ${e.message}`);
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exitGracefully(false);
        });
    }
    exitGracefully(handleKill) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a;
            this.killing = true;
            this.logger.info('Starting to exit...');
            try {
                yield ((_a = this.listener) === null || _a === void 0 ? void 0 : _a.unregister());
            }
            catch (e) {
                this.logger.error(`Could not unregister listener: ${e.message}`);
            }
            this.logger.info('Gracefully exiting hatchet worker, running tasks will attempt to finish...');
            // attempt to wait for futures to finish
            yield Promise.all(Object.values(this.futures).map(({ promise }) => promise));
            this.logger.info('Successfully finished pending tasks.');
            if (handleKill) {
                this.logger.info('Exiting hatchet worker...');
                process.exit(0);
            }
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            var _a, e_1, _b, _c;
            // ensure all workflows are registered
            yield Promise.all(this.registeredWorkflowPromises);
            try {
                this.listener = yield this.client.dispatcher.getActionListener({
                    workerName: this.name,
                    services: ['default'],
                    actions: Object.keys(this.action_registry),
                    maxRuns: this.maxRuns,
                });
                const generator = this.listener.actions();
                this.logger.info(`Worker ${this.name} listening for actions`);
                try {
                    for (var _d = true, generator_1 = __asyncValues(generator), generator_1_1; generator_1_1 = yield generator_1.next(), _a = generator_1_1.done, !_a; _d = true) {
                        _c = generator_1_1.value;
                        _d = false;
                        const action = _c;
                        this.logger.info(`Worker ${this.name} received action ${action.actionId}:${action.actionType}`);
                        void this.handleAction(action);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (!_d && !_a && (_b = generator_1.return)) yield _b.call(generator_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            catch (e) {
                // TODO TEMP this needs to be handled better
                if (this.killing) {
                    this.logger.info(`Exiting worker, ignoring error: ${e.message}`);
                    return;
                }
                this.logger.error(`Could not run worker: ${e.message}`);
                throw new hatchet_error_1.default(`Could not run worker: ${e.message}`);
            }
        });
    }
    handleAction(action) {
        return __awaiter(this, void 0, void 0, function* () {
            const type = action.actionType
                ? (0, dispatcher_1.actionTypeFromJSON)(action.actionType)
                : dispatcher_1.ActionType.START_STEP_RUN;
            if (type === dispatcher_1.ActionType.START_STEP_RUN) {
                yield this.handleStartStepRun(action);
            }
            else if (type === dispatcher_1.ActionType.CANCEL_STEP_RUN) {
                yield this.handleCancelStepRun(action);
            }
            else if (type === dispatcher_1.ActionType.START_GET_GROUP_KEY) {
                yield this.handleStartGroupKeyRun(action);
            }
            else {
                this.logger.error(`Worker ${this.name} received unknown action type ${type}`);
            }
        });
    }
}
exports.Worker = Worker;
