import { type CallContext, type CallOptions } from 'nice-grpc-common';
import * as _m0 from 'protobufjs/minimal';
export declare const protobufPackage = "";
export declare enum ActionType {
    START_STEP_RUN = 0,
    CANCEL_STEP_RUN = 1,
    START_GET_GROUP_KEY = 2,
    UNRECOGNIZED = -1
}
export declare function actionTypeFromJSON(object: any): ActionType;
export declare function actionTypeToJSON(object: ActionType): string;
export declare enum GroupKeyActionEventType {
    GROUP_KEY_EVENT_TYPE_UNKNOWN = 0,
    GROUP_KEY_EVENT_TYPE_STARTED = 1,
    GROUP_KEY_EVENT_TYPE_COMPLETED = 2,
    GROUP_KEY_EVENT_TYPE_FAILED = 3,
    UNRECOGNIZED = -1
}
export declare function groupKeyActionEventTypeFromJSON(object: any): GroupKeyActionEventType;
export declare function groupKeyActionEventTypeToJSON(object: GroupKeyActionEventType): string;
export declare enum StepActionEventType {
    STEP_EVENT_TYPE_UNKNOWN = 0,
    STEP_EVENT_TYPE_STARTED = 1,
    STEP_EVENT_TYPE_COMPLETED = 2,
    STEP_EVENT_TYPE_FAILED = 3,
    UNRECOGNIZED = -1
}
export declare function stepActionEventTypeFromJSON(object: any): StepActionEventType;
export declare function stepActionEventTypeToJSON(object: StepActionEventType): string;
export declare enum ResourceType {
    RESOURCE_TYPE_UNKNOWN = 0,
    RESOURCE_TYPE_STEP_RUN = 1,
    RESOURCE_TYPE_WORKFLOW_RUN = 2,
    UNRECOGNIZED = -1
}
export declare function resourceTypeFromJSON(object: any): ResourceType;
export declare function resourceTypeToJSON(object: ResourceType): string;
export declare enum ResourceEventType {
    RESOURCE_EVENT_TYPE_UNKNOWN = 0,
    RESOURCE_EVENT_TYPE_STARTED = 1,
    RESOURCE_EVENT_TYPE_COMPLETED = 2,
    RESOURCE_EVENT_TYPE_FAILED = 3,
    RESOURCE_EVENT_TYPE_CANCELLED = 4,
    RESOURCE_EVENT_TYPE_TIMED_OUT = 5,
    RESOURCE_EVENT_TYPE_STREAM = 6,
    UNRECOGNIZED = -1
}
export declare function resourceEventTypeFromJSON(object: any): ResourceEventType;
export declare function resourceEventTypeToJSON(object: ResourceEventType): string;
export declare enum WorkflowRunEventType {
    WORKFLOW_RUN_EVENT_TYPE_FINISHED = 0,
    UNRECOGNIZED = -1
}
export declare function workflowRunEventTypeFromJSON(object: any): WorkflowRunEventType;
export declare function workflowRunEventTypeToJSON(object: WorkflowRunEventType): string;
export interface WorkerRegisterRequest {
    /** the name of the worker */
    workerName: string;
    /** a list of actions that this worker can run */
    actions: string[];
    /** (optional) the services for this worker */
    services: string[];
    /** (optional) the max number of runs this worker can handle */
    maxRuns?: number | undefined;
}
export interface WorkerRegisterResponse {
    /** the tenant id */
    tenantId: string;
    /** the id of the worker */
    workerId: string;
    /** the name of the worker */
    workerName: string;
}
export interface AssignedAction {
    /** the tenant id */
    tenantId: string;
    /** the workflow run id (optional) */
    workflowRunId: string;
    /** the get group key run id (optional) */
    getGroupKeyRunId: string;
    /** the job id */
    jobId: string;
    /** the job name */
    jobName: string;
    /** the job run id */
    jobRunId: string;
    /** the step id */
    stepId: string;
    /** the step run id */
    stepRunId: string;
    /** the action id */
    actionId: string;
    /** the action type */
    actionType: ActionType;
    /** the action payload */
    actionPayload: string;
    /** the step name */
    stepName: string;
    /** the count number of the retry attempt */
    retryCount: number;
}
export interface WorkerListenRequest {
    /** the id of the worker */
    workerId: string;
}
export interface WorkerUnsubscribeRequest {
    /** the id of the worker */
    workerId: string;
}
export interface WorkerUnsubscribeResponse {
    /** the tenant id to unsubscribe from */
    tenantId: string;
    /** the id of the worker */
    workerId: string;
}
export interface GroupKeyActionEvent {
    /** the id of the worker */
    workerId: string;
    /** the id of the job */
    workflowRunId: string;
    getGroupKeyRunId: string;
    /** the action id */
    actionId: string;
    eventTimestamp: Date | undefined;
    /** the step event type */
    eventType: GroupKeyActionEventType;
    /** the event payload */
    eventPayload: string;
}
export interface StepActionEvent {
    /** the id of the worker */
    workerId: string;
    /** the id of the job */
    jobId: string;
    /** the job run id */
    jobRunId: string;
    /** the id of the step */
    stepId: string;
    /** the step run id */
    stepRunId: string;
    /** the action id */
    actionId: string;
    eventTimestamp: Date | undefined;
    /** the step event type */
    eventType: StepActionEventType;
    /** the event payload */
    eventPayload: string;
}
export interface ActionEventResponse {
    /** the tenant id */
    tenantId: string;
    /** the id of the worker */
    workerId: string;
}
export interface SubscribeToWorkflowEventsRequest {
    /** the id of the workflow run */
    workflowRunId: string;
}
export interface SubscribeToWorkflowRunsRequest {
    /** the id of the workflow run */
    workflowRunId: string;
}
export interface WorkflowEvent {
    /** the id of the workflow run */
    workflowRunId: string;
    resourceType: ResourceType;
    eventType: ResourceEventType;
    resourceId: string;
    eventTimestamp: Date | undefined;
    /** the event payload */
    eventPayload: string;
    /**
     * whether this is the last event for the workflow run - server
     * will hang up the connection but clients might want to case
     */
    hangup: boolean;
    /** (optional) the max number of retries this step can handle */
    stepRetries?: number | undefined;
    /** (optional) the retry count of this step */
    retryCount?: number | undefined;
}
export interface WorkflowRunEvent {
    /** the id of the workflow run */
    workflowRunId: string;
    eventType: WorkflowRunEventType;
    eventTimestamp: Date | undefined;
    results: StepRunResult[];
}
export interface StepRunResult {
    stepRunId: string;
    stepReadableId: string;
    jobRunId: string;
    error?: string | undefined;
    output?: string | undefined;
}
export interface OverridesData {
    /** the step run id */
    stepRunId: string;
    /** the path of the data to set */
    path: string;
    /** the value to set */
    value: string;
    /** the filename of the caller */
    callerFilename: string;
}
export interface OverridesDataResponse {
}
export interface HeartbeatRequest {
    /** the id of the worker */
    workerId: string;
    /** heartbeatAt is the time the worker sent the heartbeat */
    heartbeatAt: Date | undefined;
}
export interface HeartbeatResponse {
}
export interface RefreshTimeoutRequest {
    /** the id of the step run to release */
    stepRunId: string;
    incrementTimeoutBy: string;
}
export interface RefreshTimeoutResponse {
    timeoutAt: Date | undefined;
}
export interface ReleaseSlotRequest {
    /** the id of the step run to release */
    stepRunId: string;
}
export interface ReleaseSlotResponse {
}
export declare const WorkerRegisterRequest: {
    encode(message: WorkerRegisterRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkerRegisterRequest;
    fromJSON(object: any): WorkerRegisterRequest;
    toJSON(message: WorkerRegisterRequest): unknown;
    create(base?: DeepPartial<WorkerRegisterRequest>): WorkerRegisterRequest;
    fromPartial(object: DeepPartial<WorkerRegisterRequest>): WorkerRegisterRequest;
};
export declare const WorkerRegisterResponse: {
    encode(message: WorkerRegisterResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkerRegisterResponse;
    fromJSON(object: any): WorkerRegisterResponse;
    toJSON(message: WorkerRegisterResponse): unknown;
    create(base?: DeepPartial<WorkerRegisterResponse>): WorkerRegisterResponse;
    fromPartial(object: DeepPartial<WorkerRegisterResponse>): WorkerRegisterResponse;
};
export declare const AssignedAction: {
    encode(message: AssignedAction, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): AssignedAction;
    fromJSON(object: any): AssignedAction;
    toJSON(message: AssignedAction): unknown;
    create(base?: DeepPartial<AssignedAction>): AssignedAction;
    fromPartial(object: DeepPartial<AssignedAction>): AssignedAction;
};
export declare const WorkerListenRequest: {
    encode(message: WorkerListenRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkerListenRequest;
    fromJSON(object: any): WorkerListenRequest;
    toJSON(message: WorkerListenRequest): unknown;
    create(base?: DeepPartial<WorkerListenRequest>): WorkerListenRequest;
    fromPartial(object: DeepPartial<WorkerListenRequest>): WorkerListenRequest;
};
export declare const WorkerUnsubscribeRequest: {
    encode(message: WorkerUnsubscribeRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkerUnsubscribeRequest;
    fromJSON(object: any): WorkerUnsubscribeRequest;
    toJSON(message: WorkerUnsubscribeRequest): unknown;
    create(base?: DeepPartial<WorkerUnsubscribeRequest>): WorkerUnsubscribeRequest;
    fromPartial(object: DeepPartial<WorkerUnsubscribeRequest>): WorkerUnsubscribeRequest;
};
export declare const WorkerUnsubscribeResponse: {
    encode(message: WorkerUnsubscribeResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkerUnsubscribeResponse;
    fromJSON(object: any): WorkerUnsubscribeResponse;
    toJSON(message: WorkerUnsubscribeResponse): unknown;
    create(base?: DeepPartial<WorkerUnsubscribeResponse>): WorkerUnsubscribeResponse;
    fromPartial(object: DeepPartial<WorkerUnsubscribeResponse>): WorkerUnsubscribeResponse;
};
export declare const GroupKeyActionEvent: {
    encode(message: GroupKeyActionEvent, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): GroupKeyActionEvent;
    fromJSON(object: any): GroupKeyActionEvent;
    toJSON(message: GroupKeyActionEvent): unknown;
    create(base?: DeepPartial<GroupKeyActionEvent>): GroupKeyActionEvent;
    fromPartial(object: DeepPartial<GroupKeyActionEvent>): GroupKeyActionEvent;
};
export declare const StepActionEvent: {
    encode(message: StepActionEvent, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): StepActionEvent;
    fromJSON(object: any): StepActionEvent;
    toJSON(message: StepActionEvent): unknown;
    create(base?: DeepPartial<StepActionEvent>): StepActionEvent;
    fromPartial(object: DeepPartial<StepActionEvent>): StepActionEvent;
};
export declare const ActionEventResponse: {
    encode(message: ActionEventResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): ActionEventResponse;
    fromJSON(object: any): ActionEventResponse;
    toJSON(message: ActionEventResponse): unknown;
    create(base?: DeepPartial<ActionEventResponse>): ActionEventResponse;
    fromPartial(object: DeepPartial<ActionEventResponse>): ActionEventResponse;
};
export declare const SubscribeToWorkflowEventsRequest: {
    encode(message: SubscribeToWorkflowEventsRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): SubscribeToWorkflowEventsRequest;
    fromJSON(object: any): SubscribeToWorkflowEventsRequest;
    toJSON(message: SubscribeToWorkflowEventsRequest): unknown;
    create(base?: DeepPartial<SubscribeToWorkflowEventsRequest>): SubscribeToWorkflowEventsRequest;
    fromPartial(object: DeepPartial<SubscribeToWorkflowEventsRequest>): SubscribeToWorkflowEventsRequest;
};
export declare const SubscribeToWorkflowRunsRequest: {
    encode(message: SubscribeToWorkflowRunsRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): SubscribeToWorkflowRunsRequest;
    fromJSON(object: any): SubscribeToWorkflowRunsRequest;
    toJSON(message: SubscribeToWorkflowRunsRequest): unknown;
    create(base?: DeepPartial<SubscribeToWorkflowRunsRequest>): SubscribeToWorkflowRunsRequest;
    fromPartial(object: DeepPartial<SubscribeToWorkflowRunsRequest>): SubscribeToWorkflowRunsRequest;
};
export declare const WorkflowEvent: {
    encode(message: WorkflowEvent, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowEvent;
    fromJSON(object: any): WorkflowEvent;
    toJSON(message: WorkflowEvent): unknown;
    create(base?: DeepPartial<WorkflowEvent>): WorkflowEvent;
    fromPartial(object: DeepPartial<WorkflowEvent>): WorkflowEvent;
};
export declare const WorkflowRunEvent: {
    encode(message: WorkflowRunEvent, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowRunEvent;
    fromJSON(object: any): WorkflowRunEvent;
    toJSON(message: WorkflowRunEvent): unknown;
    create(base?: DeepPartial<WorkflowRunEvent>): WorkflowRunEvent;
    fromPartial(object: DeepPartial<WorkflowRunEvent>): WorkflowRunEvent;
};
export declare const StepRunResult: {
    encode(message: StepRunResult, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): StepRunResult;
    fromJSON(object: any): StepRunResult;
    toJSON(message: StepRunResult): unknown;
    create(base?: DeepPartial<StepRunResult>): StepRunResult;
    fromPartial(object: DeepPartial<StepRunResult>): StepRunResult;
};
export declare const OverridesData: {
    encode(message: OverridesData, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): OverridesData;
    fromJSON(object: any): OverridesData;
    toJSON(message: OverridesData): unknown;
    create(base?: DeepPartial<OverridesData>): OverridesData;
    fromPartial(object: DeepPartial<OverridesData>): OverridesData;
};
export declare const OverridesDataResponse: {
    encode(_: OverridesDataResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): OverridesDataResponse;
    fromJSON(_: any): OverridesDataResponse;
    toJSON(_: OverridesDataResponse): unknown;
    create(base?: DeepPartial<OverridesDataResponse>): OverridesDataResponse;
    fromPartial(_: DeepPartial<OverridesDataResponse>): OverridesDataResponse;
};
export declare const HeartbeatRequest: {
    encode(message: HeartbeatRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): HeartbeatRequest;
    fromJSON(object: any): HeartbeatRequest;
    toJSON(message: HeartbeatRequest): unknown;
    create(base?: DeepPartial<HeartbeatRequest>): HeartbeatRequest;
    fromPartial(object: DeepPartial<HeartbeatRequest>): HeartbeatRequest;
};
export declare const HeartbeatResponse: {
    encode(_: HeartbeatResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): HeartbeatResponse;
    fromJSON(_: any): HeartbeatResponse;
    toJSON(_: HeartbeatResponse): unknown;
    create(base?: DeepPartial<HeartbeatResponse>): HeartbeatResponse;
    fromPartial(_: DeepPartial<HeartbeatResponse>): HeartbeatResponse;
};
export declare const RefreshTimeoutRequest: {
    encode(message: RefreshTimeoutRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): RefreshTimeoutRequest;
    fromJSON(object: any): RefreshTimeoutRequest;
    toJSON(message: RefreshTimeoutRequest): unknown;
    create(base?: DeepPartial<RefreshTimeoutRequest>): RefreshTimeoutRequest;
    fromPartial(object: DeepPartial<RefreshTimeoutRequest>): RefreshTimeoutRequest;
};
export declare const RefreshTimeoutResponse: {
    encode(message: RefreshTimeoutResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): RefreshTimeoutResponse;
    fromJSON(object: any): RefreshTimeoutResponse;
    toJSON(message: RefreshTimeoutResponse): unknown;
    create(base?: DeepPartial<RefreshTimeoutResponse>): RefreshTimeoutResponse;
    fromPartial(object: DeepPartial<RefreshTimeoutResponse>): RefreshTimeoutResponse;
};
export declare const ReleaseSlotRequest: {
    encode(message: ReleaseSlotRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): ReleaseSlotRequest;
    fromJSON(object: any): ReleaseSlotRequest;
    toJSON(message: ReleaseSlotRequest): unknown;
    create(base?: DeepPartial<ReleaseSlotRequest>): ReleaseSlotRequest;
    fromPartial(object: DeepPartial<ReleaseSlotRequest>): ReleaseSlotRequest;
};
export declare const ReleaseSlotResponse: {
    encode(_: ReleaseSlotResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): ReleaseSlotResponse;
    fromJSON(_: any): ReleaseSlotResponse;
    toJSON(_: ReleaseSlotResponse): unknown;
    create(base?: DeepPartial<ReleaseSlotResponse>): ReleaseSlotResponse;
    fromPartial(_: DeepPartial<ReleaseSlotResponse>): ReleaseSlotResponse;
};
export type DispatcherDefinition = typeof DispatcherDefinition;
export declare const DispatcherDefinition: {
    readonly name: "Dispatcher";
    readonly fullName: "Dispatcher";
    readonly methods: {
        readonly register: {
            readonly name: "Register";
            readonly requestType: {
                encode(message: WorkerRegisterRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkerRegisterRequest;
                fromJSON(object: any): WorkerRegisterRequest;
                toJSON(message: WorkerRegisterRequest): unknown;
                create(base?: DeepPartial<WorkerRegisterRequest>): WorkerRegisterRequest;
                fromPartial(object: DeepPartial<WorkerRegisterRequest>): WorkerRegisterRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: WorkerRegisterResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkerRegisterResponse;
                fromJSON(object: any): WorkerRegisterResponse;
                toJSON(message: WorkerRegisterResponse): unknown;
                create(base?: DeepPartial<WorkerRegisterResponse>): WorkerRegisterResponse;
                fromPartial(object: DeepPartial<WorkerRegisterResponse>): WorkerRegisterResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly listen: {
            readonly name: "Listen";
            readonly requestType: {
                encode(message: WorkerListenRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkerListenRequest;
                fromJSON(object: any): WorkerListenRequest;
                toJSON(message: WorkerListenRequest): unknown;
                create(base?: DeepPartial<WorkerListenRequest>): WorkerListenRequest;
                fromPartial(object: DeepPartial<WorkerListenRequest>): WorkerListenRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: AssignedAction, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): AssignedAction;
                fromJSON(object: any): AssignedAction;
                toJSON(message: AssignedAction): unknown;
                create(base?: DeepPartial<AssignedAction>): AssignedAction;
                fromPartial(object: DeepPartial<AssignedAction>): AssignedAction;
            };
            readonly responseStream: true;
            readonly options: {};
        };
        /**
         * ListenV2 is like listen, but implementation does not include heartbeats. This should only used by SDKs
         * against engine version v0.18.1+
         */
        readonly listenV2: {
            readonly name: "ListenV2";
            readonly requestType: {
                encode(message: WorkerListenRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkerListenRequest;
                fromJSON(object: any): WorkerListenRequest;
                toJSON(message: WorkerListenRequest): unknown;
                create(base?: DeepPartial<WorkerListenRequest>): WorkerListenRequest;
                fromPartial(object: DeepPartial<WorkerListenRequest>): WorkerListenRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: AssignedAction, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): AssignedAction;
                fromJSON(object: any): AssignedAction;
                toJSON(message: AssignedAction): unknown;
                create(base?: DeepPartial<AssignedAction>): AssignedAction;
                fromPartial(object: DeepPartial<AssignedAction>): AssignedAction;
            };
            readonly responseStream: true;
            readonly options: {};
        };
        /** Heartbeat is a method for workers to send heartbeats to the dispatcher */
        readonly heartbeat: {
            readonly name: "Heartbeat";
            readonly requestType: {
                encode(message: HeartbeatRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): HeartbeatRequest;
                fromJSON(object: any): HeartbeatRequest;
                toJSON(message: HeartbeatRequest): unknown;
                create(base?: DeepPartial<HeartbeatRequest>): HeartbeatRequest;
                fromPartial(object: DeepPartial<HeartbeatRequest>): HeartbeatRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(_: HeartbeatResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): HeartbeatResponse;
                fromJSON(_: any): HeartbeatResponse;
                toJSON(_: HeartbeatResponse): unknown;
                create(base?: DeepPartial<HeartbeatResponse>): HeartbeatResponse;
                fromPartial(_: DeepPartial<HeartbeatResponse>): HeartbeatResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly subscribeToWorkflowEvents: {
            readonly name: "SubscribeToWorkflowEvents";
            readonly requestType: {
                encode(message: SubscribeToWorkflowEventsRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): SubscribeToWorkflowEventsRequest;
                fromJSON(object: any): SubscribeToWorkflowEventsRequest;
                toJSON(message: SubscribeToWorkflowEventsRequest): unknown;
                create(base?: DeepPartial<SubscribeToWorkflowEventsRequest>): SubscribeToWorkflowEventsRequest;
                fromPartial(object: DeepPartial<SubscribeToWorkflowEventsRequest>): SubscribeToWorkflowEventsRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: WorkflowEvent, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowEvent;
                fromJSON(object: any): WorkflowEvent;
                toJSON(message: WorkflowEvent): unknown;
                create(base?: DeepPartial<WorkflowEvent>): WorkflowEvent;
                fromPartial(object: DeepPartial<WorkflowEvent>): WorkflowEvent;
            };
            readonly responseStream: true;
            readonly options: {};
        };
        readonly subscribeToWorkflowRuns: {
            readonly name: "SubscribeToWorkflowRuns";
            readonly requestType: {
                encode(message: SubscribeToWorkflowRunsRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): SubscribeToWorkflowRunsRequest;
                fromJSON(object: any): SubscribeToWorkflowRunsRequest;
                toJSON(message: SubscribeToWorkflowRunsRequest): unknown;
                create(base?: DeepPartial<SubscribeToWorkflowRunsRequest>): SubscribeToWorkflowRunsRequest;
                fromPartial(object: DeepPartial<SubscribeToWorkflowRunsRequest>): SubscribeToWorkflowRunsRequest;
            };
            readonly requestStream: true;
            readonly responseType: {
                encode(message: WorkflowRunEvent, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowRunEvent;
                fromJSON(object: any): WorkflowRunEvent;
                toJSON(message: WorkflowRunEvent): unknown;
                create(base?: DeepPartial<WorkflowRunEvent>): WorkflowRunEvent;
                fromPartial(object: DeepPartial<WorkflowRunEvent>): WorkflowRunEvent;
            };
            readonly responseStream: true;
            readonly options: {};
        };
        readonly sendStepActionEvent: {
            readonly name: "SendStepActionEvent";
            readonly requestType: {
                encode(message: StepActionEvent, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): StepActionEvent;
                fromJSON(object: any): StepActionEvent;
                toJSON(message: StepActionEvent): unknown;
                create(base?: DeepPartial<StepActionEvent>): StepActionEvent;
                fromPartial(object: DeepPartial<StepActionEvent>): StepActionEvent;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: ActionEventResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): ActionEventResponse;
                fromJSON(object: any): ActionEventResponse;
                toJSON(message: ActionEventResponse): unknown;
                create(base?: DeepPartial<ActionEventResponse>): ActionEventResponse;
                fromPartial(object: DeepPartial<ActionEventResponse>): ActionEventResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly sendGroupKeyActionEvent: {
            readonly name: "SendGroupKeyActionEvent";
            readonly requestType: {
                encode(message: GroupKeyActionEvent, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): GroupKeyActionEvent;
                fromJSON(object: any): GroupKeyActionEvent;
                toJSON(message: GroupKeyActionEvent): unknown;
                create(base?: DeepPartial<GroupKeyActionEvent>): GroupKeyActionEvent;
                fromPartial(object: DeepPartial<GroupKeyActionEvent>): GroupKeyActionEvent;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: ActionEventResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): ActionEventResponse;
                fromJSON(object: any): ActionEventResponse;
                toJSON(message: ActionEventResponse): unknown;
                create(base?: DeepPartial<ActionEventResponse>): ActionEventResponse;
                fromPartial(object: DeepPartial<ActionEventResponse>): ActionEventResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly putOverridesData: {
            readonly name: "PutOverridesData";
            readonly requestType: {
                encode(message: OverridesData, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): OverridesData;
                fromJSON(object: any): OverridesData;
                toJSON(message: OverridesData): unknown;
                create(base?: DeepPartial<OverridesData>): OverridesData;
                fromPartial(object: DeepPartial<OverridesData>): OverridesData;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(_: OverridesDataResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): OverridesDataResponse;
                fromJSON(_: any): OverridesDataResponse;
                toJSON(_: OverridesDataResponse): unknown;
                create(base?: DeepPartial<OverridesDataResponse>): OverridesDataResponse;
                fromPartial(_: DeepPartial<OverridesDataResponse>): OverridesDataResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly unsubscribe: {
            readonly name: "Unsubscribe";
            readonly requestType: {
                encode(message: WorkerUnsubscribeRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkerUnsubscribeRequest;
                fromJSON(object: any): WorkerUnsubscribeRequest;
                toJSON(message: WorkerUnsubscribeRequest): unknown;
                create(base?: DeepPartial<WorkerUnsubscribeRequest>): WorkerUnsubscribeRequest;
                fromPartial(object: DeepPartial<WorkerUnsubscribeRequest>): WorkerUnsubscribeRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: WorkerUnsubscribeResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkerUnsubscribeResponse;
                fromJSON(object: any): WorkerUnsubscribeResponse;
                toJSON(message: WorkerUnsubscribeResponse): unknown;
                create(base?: DeepPartial<WorkerUnsubscribeResponse>): WorkerUnsubscribeResponse;
                fromPartial(object: DeepPartial<WorkerUnsubscribeResponse>): WorkerUnsubscribeResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly refreshTimeout: {
            readonly name: "RefreshTimeout";
            readonly requestType: {
                encode(message: RefreshTimeoutRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): RefreshTimeoutRequest;
                fromJSON(object: any): RefreshTimeoutRequest;
                toJSON(message: RefreshTimeoutRequest): unknown;
                create(base?: DeepPartial<RefreshTimeoutRequest>): RefreshTimeoutRequest;
                fromPartial(object: DeepPartial<RefreshTimeoutRequest>): RefreshTimeoutRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: RefreshTimeoutResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): RefreshTimeoutResponse;
                fromJSON(object: any): RefreshTimeoutResponse;
                toJSON(message: RefreshTimeoutResponse): unknown;
                create(base?: DeepPartial<RefreshTimeoutResponse>): RefreshTimeoutResponse;
                fromPartial(object: DeepPartial<RefreshTimeoutResponse>): RefreshTimeoutResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly releaseSlot: {
            readonly name: "ReleaseSlot";
            readonly requestType: {
                encode(message: ReleaseSlotRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): ReleaseSlotRequest;
                fromJSON(object: any): ReleaseSlotRequest;
                toJSON(message: ReleaseSlotRequest): unknown;
                create(base?: DeepPartial<ReleaseSlotRequest>): ReleaseSlotRequest;
                fromPartial(object: DeepPartial<ReleaseSlotRequest>): ReleaseSlotRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(_: ReleaseSlotResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): ReleaseSlotResponse;
                fromJSON(_: any): ReleaseSlotResponse;
                toJSON(_: ReleaseSlotResponse): unknown;
                create(base?: DeepPartial<ReleaseSlotResponse>): ReleaseSlotResponse;
                fromPartial(_: DeepPartial<ReleaseSlotResponse>): ReleaseSlotResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
    };
};
export interface DispatcherServiceImplementation<CallContextExt = {}> {
    register(request: WorkerRegisterRequest, context: CallContext & CallContextExt): Promise<DeepPartial<WorkerRegisterResponse>>;
    listen(request: WorkerListenRequest, context: CallContext & CallContextExt): ServerStreamingMethodResult<DeepPartial<AssignedAction>>;
    /**
     * ListenV2 is like listen, but implementation does not include heartbeats. This should only used by SDKs
     * against engine version v0.18.1+
     */
    listenV2(request: WorkerListenRequest, context: CallContext & CallContextExt): ServerStreamingMethodResult<DeepPartial<AssignedAction>>;
    /** Heartbeat is a method for workers to send heartbeats to the dispatcher */
    heartbeat(request: HeartbeatRequest, context: CallContext & CallContextExt): Promise<DeepPartial<HeartbeatResponse>>;
    subscribeToWorkflowEvents(request: SubscribeToWorkflowEventsRequest, context: CallContext & CallContextExt): ServerStreamingMethodResult<DeepPartial<WorkflowEvent>>;
    subscribeToWorkflowRuns(request: AsyncIterable<SubscribeToWorkflowRunsRequest>, context: CallContext & CallContextExt): ServerStreamingMethodResult<DeepPartial<WorkflowRunEvent>>;
    sendStepActionEvent(request: StepActionEvent, context: CallContext & CallContextExt): Promise<DeepPartial<ActionEventResponse>>;
    sendGroupKeyActionEvent(request: GroupKeyActionEvent, context: CallContext & CallContextExt): Promise<DeepPartial<ActionEventResponse>>;
    putOverridesData(request: OverridesData, context: CallContext & CallContextExt): Promise<DeepPartial<OverridesDataResponse>>;
    unsubscribe(request: WorkerUnsubscribeRequest, context: CallContext & CallContextExt): Promise<DeepPartial<WorkerUnsubscribeResponse>>;
    refreshTimeout(request: RefreshTimeoutRequest, context: CallContext & CallContextExt): Promise<DeepPartial<RefreshTimeoutResponse>>;
    releaseSlot(request: ReleaseSlotRequest, context: CallContext & CallContextExt): Promise<DeepPartial<ReleaseSlotResponse>>;
}
export interface DispatcherClient<CallOptionsExt = {}> {
    register(request: DeepPartial<WorkerRegisterRequest>, options?: CallOptions & CallOptionsExt): Promise<WorkerRegisterResponse>;
    listen(request: DeepPartial<WorkerListenRequest>, options?: CallOptions & CallOptionsExt): AsyncIterable<AssignedAction>;
    /**
     * ListenV2 is like listen, but implementation does not include heartbeats. This should only used by SDKs
     * against engine version v0.18.1+
     */
    listenV2(request: DeepPartial<WorkerListenRequest>, options?: CallOptions & CallOptionsExt): AsyncIterable<AssignedAction>;
    /** Heartbeat is a method for workers to send heartbeats to the dispatcher */
    heartbeat(request: DeepPartial<HeartbeatRequest>, options?: CallOptions & CallOptionsExt): Promise<HeartbeatResponse>;
    subscribeToWorkflowEvents(request: DeepPartial<SubscribeToWorkflowEventsRequest>, options?: CallOptions & CallOptionsExt): AsyncIterable<WorkflowEvent>;
    subscribeToWorkflowRuns(request: AsyncIterable<DeepPartial<SubscribeToWorkflowRunsRequest>>, options?: CallOptions & CallOptionsExt): AsyncIterable<WorkflowRunEvent>;
    sendStepActionEvent(request: DeepPartial<StepActionEvent>, options?: CallOptions & CallOptionsExt): Promise<ActionEventResponse>;
    sendGroupKeyActionEvent(request: DeepPartial<GroupKeyActionEvent>, options?: CallOptions & CallOptionsExt): Promise<ActionEventResponse>;
    putOverridesData(request: DeepPartial<OverridesData>, options?: CallOptions & CallOptionsExt): Promise<OverridesDataResponse>;
    unsubscribe(request: DeepPartial<WorkerUnsubscribeRequest>, options?: CallOptions & CallOptionsExt): Promise<WorkerUnsubscribeResponse>;
    refreshTimeout(request: DeepPartial<RefreshTimeoutRequest>, options?: CallOptions & CallOptionsExt): Promise<RefreshTimeoutResponse>;
    releaseSlot(request: DeepPartial<ReleaseSlotRequest>, options?: CallOptions & CallOptionsExt): Promise<ReleaseSlotResponse>;
}
type Builtin = Date | Function | Uint8Array | string | number | boolean | undefined;
export type DeepPartial<T> = T extends Builtin ? T : T extends globalThis.Array<infer U> ? globalThis.Array<DeepPartial<U>> : T extends ReadonlyArray<infer U> ? ReadonlyArray<DeepPartial<U>> : T extends {} ? {
    [K in keyof T]?: DeepPartial<T[K]>;
} : Partial<T>;
export type ServerStreamingMethodResult<Response> = {
    [Symbol.asyncIterator](): AsyncIterator<Response, void>;
};
export {};
