import { type CallContext, type CallOptions } from 'nice-grpc-common';
import * as _m0 from 'protobufjs/minimal';
export declare const protobufPackage = "";
export declare enum ConcurrencyLimitStrategy {
    CANCEL_IN_PROGRESS = 0,
    DROP_NEWEST = 1,
    QUEUE_NEWEST = 2,
    GROUP_ROUND_ROBIN = 3,
    UNRECOGNIZED = -1
}
export declare function concurrencyLimitStrategyFromJSON(object: any): ConcurrencyLimitStrategy;
export declare function concurrencyLimitStrategyToJSON(object: ConcurrencyLimitStrategy): string;
export declare enum RateLimitDuration {
    SECOND = 0,
    MINUTE = 1,
    HOUR = 2,
    DAY = 3,
    WEEK = 4,
    MONTH = 5,
    YEAR = 6,
    UNRECOGNIZED = -1
}
export declare function rateLimitDurationFromJSON(object: any): RateLimitDuration;
export declare function rateLimitDurationToJSON(object: RateLimitDuration): string;
export interface PutWorkflowRequest {
    opts: CreateWorkflowVersionOpts | undefined;
}
/** CreateWorkflowVersionOpts represents options to create a workflow version. */
export interface CreateWorkflowVersionOpts {
    /** (required) the workflow name */
    name: string;
    /** (optional) the workflow description */
    description: string;
    /** (required) the workflow version */
    version: string;
    /** (optional) event triggers for the workflow */
    eventTriggers: string[];
    /** (optional) cron triggers for the workflow */
    cronTriggers: string[];
    /** (optional) scheduled triggers for the workflow */
    scheduledTriggers: Date[];
    /** (required) the workflow jobs */
    jobs: CreateWorkflowJobOpts[];
    /** (optional) the workflow concurrency options */
    concurrency: WorkflowConcurrencyOpts | undefined;
    /** (optional) the timeout for the schedule */
    scheduleTimeout?: string | undefined;
    /** (optional) the input for the cron trigger */
    cronInput?: string | undefined;
    /** (optional) the job to run on failure */
    onFailureJob?: CreateWorkflowJobOpts | undefined;
}
export interface WorkflowConcurrencyOpts {
    /** (required) the action id for getting the concurrency group */
    action: string;
    /** (optional) the maximum number of concurrent workflow runs, default 1 */
    maxRuns: number;
    /** (optional) the strategy to use when the concurrency limit is reached, default CANCEL_IN_PROGRESS */
    limitStrategy: ConcurrencyLimitStrategy;
}
/** CreateWorkflowJobOpts represents options to create a workflow job. */
export interface CreateWorkflowJobOpts {
    /** (required) the job name */
    name: string;
    /** (optional) the job description */
    description: string;
    /** (required) the job steps */
    steps: CreateWorkflowStepOpts[];
}
/** CreateWorkflowStepOpts represents options to create a workflow step. */
export interface CreateWorkflowStepOpts {
    /** (required) the step name */
    readableId: string;
    /** (required) the step action id */
    action: string;
    /** (optional) the step timeout */
    timeout: string;
    /** (optional) the step inputs, assuming string representation of JSON */
    inputs: string;
    /** (optional) the step parents. if none are passed in, this is a root step */
    parents: string[];
    /** (optional) the custom step user data, assuming string representation of JSON */
    userData: string;
    /** (optional) the number of retries for the step, default 0 */
    retries: number;
    /** (optional) the rate limits for the step */
    rateLimits: CreateStepRateLimit[];
}
export interface CreateStepRateLimit {
    /** (required) the key for the rate limit */
    key: string;
    /** (required) the number of units this step consumes */
    units: number;
}
/** ListWorkflowsRequest is the request for ListWorkflows. */
export interface ListWorkflowsRequest {
}
export interface ScheduleWorkflowRequest {
    name: string;
    schedules: Date[];
    /** (optional) the input data for the workflow */
    input: string;
    /** (optional) the parent workflow run id */
    parentId?: string | undefined;
    /** (optional) the parent step run id */
    parentStepRunId?: string | undefined;
    /**
     * (optional) the index of the child workflow. if this is set, matches on the index or the
     * child key will be a no-op, even if the schedule has changed.
     */
    childIndex?: number | undefined;
    /**
     * (optional) the key for the child. if this is set, matches on the index or the
     * child key will be a no-op, even if the schedule has changed.
     */
    childKey?: string | undefined;
}
/** WorkflowVersion represents the WorkflowVersion model. */
export interface WorkflowVersion {
    id: string;
    createdAt: Date | undefined;
    updatedAt: Date | undefined;
    version: string;
    order: number;
    workflowId: string;
}
/** WorkflowTriggerEventRef represents the WorkflowTriggerEventRef model. */
export interface WorkflowTriggerEventRef {
    parentId: string;
    eventKey: string;
}
/** WorkflowTriggerCronRef represents the WorkflowTriggerCronRef model. */
export interface WorkflowTriggerCronRef {
    parentId: string;
    cron: string;
}
export interface TriggerWorkflowRequest {
    name: string;
    /** (optional) the input data for the workflow */
    input: string;
    /** (optional) the parent workflow run id */
    parentId?: string | undefined;
    /** (optional) the parent step run id */
    parentStepRunId?: string | undefined;
    /**
     * (optional) the index of the child workflow. if this is set, matches on the index or the
     * child key will return an existing workflow run if the parent id, parent step run id, and
     * child index/key match an existing workflow run.
     */
    childIndex?: number | undefined;
    /**
     * (optional) the key for the child. if this is set, matches on the index or the
     * child key will return an existing workflow run if the parent id, parent step run id, and
     * child index/key match an existing workflow run.
     */
    childKey?: string | undefined;
    /** (optional) additional metadata for the workflow */
    additionalMetadata?: string | undefined;
}
export interface TriggerWorkflowResponse {
    workflowRunId: string;
}
export interface PutRateLimitRequest {
    /** (required) the global key for the rate limit */
    key: string;
    /** (required) the max limit for the rate limit (per unit of time) */
    limit: number;
    /** (required) the duration of time for the rate limit (second|minute|hour) */
    duration: RateLimitDuration;
}
export interface PutRateLimitResponse {
}
export declare const PutWorkflowRequest: {
    encode(message: PutWorkflowRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): PutWorkflowRequest;
    fromJSON(object: any): PutWorkflowRequest;
    toJSON(message: PutWorkflowRequest): unknown;
    create(base?: DeepPartial<PutWorkflowRequest>): PutWorkflowRequest;
    fromPartial(object: DeepPartial<PutWorkflowRequest>): PutWorkflowRequest;
};
export declare const CreateWorkflowVersionOpts: {
    encode(message: CreateWorkflowVersionOpts, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): CreateWorkflowVersionOpts;
    fromJSON(object: any): CreateWorkflowVersionOpts;
    toJSON(message: CreateWorkflowVersionOpts): unknown;
    create(base?: DeepPartial<CreateWorkflowVersionOpts>): CreateWorkflowVersionOpts;
    fromPartial(object: DeepPartial<CreateWorkflowVersionOpts>): CreateWorkflowVersionOpts;
};
export declare const WorkflowConcurrencyOpts: {
    encode(message: WorkflowConcurrencyOpts, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowConcurrencyOpts;
    fromJSON(object: any): WorkflowConcurrencyOpts;
    toJSON(message: WorkflowConcurrencyOpts): unknown;
    create(base?: DeepPartial<WorkflowConcurrencyOpts>): WorkflowConcurrencyOpts;
    fromPartial(object: DeepPartial<WorkflowConcurrencyOpts>): WorkflowConcurrencyOpts;
};
export declare const CreateWorkflowJobOpts: {
    encode(message: CreateWorkflowJobOpts, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): CreateWorkflowJobOpts;
    fromJSON(object: any): CreateWorkflowJobOpts;
    toJSON(message: CreateWorkflowJobOpts): unknown;
    create(base?: DeepPartial<CreateWorkflowJobOpts>): CreateWorkflowJobOpts;
    fromPartial(object: DeepPartial<CreateWorkflowJobOpts>): CreateWorkflowJobOpts;
};
export declare const CreateWorkflowStepOpts: {
    encode(message: CreateWorkflowStepOpts, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): CreateWorkflowStepOpts;
    fromJSON(object: any): CreateWorkflowStepOpts;
    toJSON(message: CreateWorkflowStepOpts): unknown;
    create(base?: DeepPartial<CreateWorkflowStepOpts>): CreateWorkflowStepOpts;
    fromPartial(object: DeepPartial<CreateWorkflowStepOpts>): CreateWorkflowStepOpts;
};
export declare const CreateStepRateLimit: {
    encode(message: CreateStepRateLimit, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): CreateStepRateLimit;
    fromJSON(object: any): CreateStepRateLimit;
    toJSON(message: CreateStepRateLimit): unknown;
    create(base?: DeepPartial<CreateStepRateLimit>): CreateStepRateLimit;
    fromPartial(object: DeepPartial<CreateStepRateLimit>): CreateStepRateLimit;
};
export declare const ListWorkflowsRequest: {
    encode(_: ListWorkflowsRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): ListWorkflowsRequest;
    fromJSON(_: any): ListWorkflowsRequest;
    toJSON(_: ListWorkflowsRequest): unknown;
    create(base?: DeepPartial<ListWorkflowsRequest>): ListWorkflowsRequest;
    fromPartial(_: DeepPartial<ListWorkflowsRequest>): ListWorkflowsRequest;
};
export declare const ScheduleWorkflowRequest: {
    encode(message: ScheduleWorkflowRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): ScheduleWorkflowRequest;
    fromJSON(object: any): ScheduleWorkflowRequest;
    toJSON(message: ScheduleWorkflowRequest): unknown;
    create(base?: DeepPartial<ScheduleWorkflowRequest>): ScheduleWorkflowRequest;
    fromPartial(object: DeepPartial<ScheduleWorkflowRequest>): ScheduleWorkflowRequest;
};
export declare const WorkflowVersion: {
    encode(message: WorkflowVersion, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowVersion;
    fromJSON(object: any): WorkflowVersion;
    toJSON(message: WorkflowVersion): unknown;
    create(base?: DeepPartial<WorkflowVersion>): WorkflowVersion;
    fromPartial(object: DeepPartial<WorkflowVersion>): WorkflowVersion;
};
export declare const WorkflowTriggerEventRef: {
    encode(message: WorkflowTriggerEventRef, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowTriggerEventRef;
    fromJSON(object: any): WorkflowTriggerEventRef;
    toJSON(message: WorkflowTriggerEventRef): unknown;
    create(base?: DeepPartial<WorkflowTriggerEventRef>): WorkflowTriggerEventRef;
    fromPartial(object: DeepPartial<WorkflowTriggerEventRef>): WorkflowTriggerEventRef;
};
export declare const WorkflowTriggerCronRef: {
    encode(message: WorkflowTriggerCronRef, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowTriggerCronRef;
    fromJSON(object: any): WorkflowTriggerCronRef;
    toJSON(message: WorkflowTriggerCronRef): unknown;
    create(base?: DeepPartial<WorkflowTriggerCronRef>): WorkflowTriggerCronRef;
    fromPartial(object: DeepPartial<WorkflowTriggerCronRef>): WorkflowTriggerCronRef;
};
export declare const TriggerWorkflowRequest: {
    encode(message: TriggerWorkflowRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): TriggerWorkflowRequest;
    fromJSON(object: any): TriggerWorkflowRequest;
    toJSON(message: TriggerWorkflowRequest): unknown;
    create(base?: DeepPartial<TriggerWorkflowRequest>): TriggerWorkflowRequest;
    fromPartial(object: DeepPartial<TriggerWorkflowRequest>): TriggerWorkflowRequest;
};
export declare const TriggerWorkflowResponse: {
    encode(message: TriggerWorkflowResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): TriggerWorkflowResponse;
    fromJSON(object: any): TriggerWorkflowResponse;
    toJSON(message: TriggerWorkflowResponse): unknown;
    create(base?: DeepPartial<TriggerWorkflowResponse>): TriggerWorkflowResponse;
    fromPartial(object: DeepPartial<TriggerWorkflowResponse>): TriggerWorkflowResponse;
};
export declare const PutRateLimitRequest: {
    encode(message: PutRateLimitRequest, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): PutRateLimitRequest;
    fromJSON(object: any): PutRateLimitRequest;
    toJSON(message: PutRateLimitRequest): unknown;
    create(base?: DeepPartial<PutRateLimitRequest>): PutRateLimitRequest;
    fromPartial(object: DeepPartial<PutRateLimitRequest>): PutRateLimitRequest;
};
export declare const PutRateLimitResponse: {
    encode(_: PutRateLimitResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): PutRateLimitResponse;
    fromJSON(_: any): PutRateLimitResponse;
    toJSON(_: PutRateLimitResponse): unknown;
    create(base?: DeepPartial<PutRateLimitResponse>): PutRateLimitResponse;
    fromPartial(_: DeepPartial<PutRateLimitResponse>): PutRateLimitResponse;
};
/** WorkflowService represents a set of RPCs for managing workflows. */
export type WorkflowServiceDefinition = typeof WorkflowServiceDefinition;
export declare const WorkflowServiceDefinition: {
    readonly name: "WorkflowService";
    readonly fullName: "WorkflowService";
    readonly methods: {
        readonly putWorkflow: {
            readonly name: "PutWorkflow";
            readonly requestType: {
                encode(message: PutWorkflowRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): PutWorkflowRequest;
                fromJSON(object: any): PutWorkflowRequest;
                toJSON(message: PutWorkflowRequest): unknown;
                create(base?: DeepPartial<PutWorkflowRequest>): PutWorkflowRequest;
                fromPartial(object: DeepPartial<PutWorkflowRequest>): PutWorkflowRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: WorkflowVersion, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowVersion;
                fromJSON(object: any): WorkflowVersion;
                toJSON(message: WorkflowVersion): unknown;
                create(base?: DeepPartial<WorkflowVersion>): WorkflowVersion;
                fromPartial(object: DeepPartial<WorkflowVersion>): WorkflowVersion;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly scheduleWorkflow: {
            readonly name: "ScheduleWorkflow";
            readonly requestType: {
                encode(message: ScheduleWorkflowRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): ScheduleWorkflowRequest;
                fromJSON(object: any): ScheduleWorkflowRequest;
                toJSON(message: ScheduleWorkflowRequest): unknown;
                create(base?: DeepPartial<ScheduleWorkflowRequest>): ScheduleWorkflowRequest;
                fromPartial(object: DeepPartial<ScheduleWorkflowRequest>): ScheduleWorkflowRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: WorkflowVersion, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): WorkflowVersion;
                fromJSON(object: any): WorkflowVersion;
                toJSON(message: WorkflowVersion): unknown;
                create(base?: DeepPartial<WorkflowVersion>): WorkflowVersion;
                fromPartial(object: DeepPartial<WorkflowVersion>): WorkflowVersion;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly triggerWorkflow: {
            readonly name: "TriggerWorkflow";
            readonly requestType: {
                encode(message: TriggerWorkflowRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): TriggerWorkflowRequest;
                fromJSON(object: any): TriggerWorkflowRequest;
                toJSON(message: TriggerWorkflowRequest): unknown;
                create(base?: DeepPartial<TriggerWorkflowRequest>): TriggerWorkflowRequest;
                fromPartial(object: DeepPartial<TriggerWorkflowRequest>): TriggerWorkflowRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(message: TriggerWorkflowResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): TriggerWorkflowResponse;
                fromJSON(object: any): TriggerWorkflowResponse;
                toJSON(message: TriggerWorkflowResponse): unknown;
                create(base?: DeepPartial<TriggerWorkflowResponse>): TriggerWorkflowResponse;
                fromPartial(object: DeepPartial<TriggerWorkflowResponse>): TriggerWorkflowResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
        readonly putRateLimit: {
            readonly name: "PutRateLimit";
            readonly requestType: {
                encode(message: PutRateLimitRequest, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): PutRateLimitRequest;
                fromJSON(object: any): PutRateLimitRequest;
                toJSON(message: PutRateLimitRequest): unknown;
                create(base?: DeepPartial<PutRateLimitRequest>): PutRateLimitRequest;
                fromPartial(object: DeepPartial<PutRateLimitRequest>): PutRateLimitRequest;
            };
            readonly requestStream: false;
            readonly responseType: {
                encode(_: PutRateLimitResponse, writer?: _m0.Writer): _m0.Writer;
                decode(input: _m0.Reader | Uint8Array, length?: number): PutRateLimitResponse;
                fromJSON(_: any): PutRateLimitResponse;
                toJSON(_: PutRateLimitResponse): unknown;
                create(base?: DeepPartial<PutRateLimitResponse>): PutRateLimitResponse;
                fromPartial(_: DeepPartial<PutRateLimitResponse>): PutRateLimitResponse;
            };
            readonly responseStream: false;
            readonly options: {};
        };
    };
};
export interface WorkflowServiceImplementation<CallContextExt = {}> {
    putWorkflow(request: PutWorkflowRequest, context: CallContext & CallContextExt): Promise<DeepPartial<WorkflowVersion>>;
    scheduleWorkflow(request: ScheduleWorkflowRequest, context: CallContext & CallContextExt): Promise<DeepPartial<WorkflowVersion>>;
    triggerWorkflow(request: TriggerWorkflowRequest, context: CallContext & CallContextExt): Promise<DeepPartial<TriggerWorkflowResponse>>;
    putRateLimit(request: PutRateLimitRequest, context: CallContext & CallContextExt): Promise<DeepPartial<PutRateLimitResponse>>;
}
export interface WorkflowServiceClient<CallOptionsExt = {}> {
    putWorkflow(request: DeepPartial<PutWorkflowRequest>, options?: CallOptions & CallOptionsExt): Promise<WorkflowVersion>;
    scheduleWorkflow(request: DeepPartial<ScheduleWorkflowRequest>, options?: CallOptions & CallOptionsExt): Promise<WorkflowVersion>;
    triggerWorkflow(request: DeepPartial<TriggerWorkflowRequest>, options?: CallOptions & CallOptionsExt): Promise<TriggerWorkflowResponse>;
    putRateLimit(request: DeepPartial<PutRateLimitRequest>, options?: CallOptions & CallOptionsExt): Promise<PutRateLimitResponse>;
}
type Builtin = Date | Function | Uint8Array | string | number | boolean | undefined;
export type DeepPartial<T> = T extends Builtin ? T : T extends globalThis.Array<infer U> ? globalThis.Array<DeepPartial<U>> : T extends ReadonlyArray<infer U> ? ReadonlyArray<DeepPartial<U>> : T extends {} ? {
    [K in keyof T]?: DeepPartial<T[K]>;
} : Partial<T>;
export {};
