import * as z from 'zod';
import { Action } from './clients/dispatcher/action-listener';
import { LogLevel } from './clients/event/event-client';
import { Logger } from './util/logger';
import { HatchetClient } from './clients/hatchet-client';
import { WorkflowRunEvent } from './protoc/dispatcher';
export declare const CreateRateLimitSchema: z.ZodObject<{
    key: z.ZodString;
    units: z.ZodNumber;
}, "strip", z.ZodTypeAny, {
    key: string;
    units: number;
}, {
    key: string;
    units: number;
}>;
export declare const CreateStepSchema: z.ZodObject<{
    name: z.ZodString;
    parents: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
    timeout: z.ZodOptional<z.ZodString>;
    retries: z.ZodOptional<z.ZodNumber>;
    rate_limits: z.ZodOptional<z.ZodArray<z.ZodObject<{
        key: z.ZodString;
        units: z.ZodNumber;
    }, "strip", z.ZodTypeAny, {
        key: string;
        units: number;
    }, {
        key: string;
        units: number;
    }>, "many">>;
}, "strip", z.ZodTypeAny, {
    name: string;
    timeout?: string | undefined;
    parents?: string[] | undefined;
    retries?: number | undefined;
    rate_limits?: {
        key: string;
        units: number;
    }[] | undefined;
}, {
    name: string;
    timeout?: string | undefined;
    parents?: string[] | undefined;
    retries?: number | undefined;
    rate_limits?: {
        key: string;
        units: number;
    }[] | undefined;
}>;
type JSONPrimitive = string | number | boolean | null | Array<JSONPrimitive>;
export type NextStep = {
    [key: string]: NextStep | JSONPrimitive | Array<NextStep>;
};
interface ContextData<T, K> {
    input: T;
    parents: Record<string, any>;
    triggered_by: string;
    user_data: K;
}
declare class ChildWorkflowRef<T> {
    workflowRunId: Promise<string>;
    parentWorkflowRunId: string;
    client: HatchetClient;
    constructor(workflowRunId: Promise<string>, parentWorkflowRunId: string, client: HatchetClient);
    stream(): Promise<AsyncGenerator<WorkflowRunEvent, void, unknown>>;
    result(): Promise<T>;
    toJSON(): Promise<string>;
}
export declare class Context<T, K> {
    data: ContextData<T, K>;
    input: T;
    controller: AbortController;
    action: Action;
    client: HatchetClient;
    overridesData: Record<string, any>;
    logger: Logger;
    spawnIndex: number;
    constructor(action: Action, client: HatchetClient);
    stepOutput(step: string): NextStep;
    triggeredByEvent(): boolean;
    workflowInput(): T;
    workflowName(): string;
    userData(): K;
    stepName(): string;
    workflowRunId(): string;
    playground(name: string, defaultValue?: string): string;
    log(message: string, level?: LogLevel): void;
    /**
     * Refreshes the timeout for the current step.
     * @param incrementBy - The interval by which to increment the timeout.
     *                     The interval should be specified in the format of '10s' for 10 seconds,
     *                     '1m' for 1 minute, or '1d' for 1 day.
     */
    refreshTimeout(incrementBy: string): Promise<void>;
    releaseSlot(): Promise<void>;
    putStream(data: string | Uint8Array): Promise<void>;
    spawnWorkflow<P = unknown, Q = unknown>(workflowName: string, input: Q, key?: string): ChildWorkflowRef<P>;
}
export type StepRunFunction<T, K> = (ctx: Context<T, K>) => Promise<NextStep | void> | NextStep | void;
export interface CreateStep<T, K> extends z.infer<typeof CreateStepSchema> {
    run: StepRunFunction<T, K>;
}
export {};
