"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Context = exports.CreateStepSchema = exports.CreateRateLimitSchema = void 0;
/* eslint-disable max-classes-per-file */
const hatchet_error_1 = __importDefault(require("./util/errors/hatchet-error"));
const z = __importStar(require("zod"));
const workflow_1 = require("./workflow");
const logger_1 = require("./util/logger");
const parse_1 = require("./util/parse");
const dispatcher_1 = require("./protoc/dispatcher");
exports.CreateRateLimitSchema = z.object({
    key: z.string(),
    units: z.number().min(1),
});
exports.CreateStepSchema = z.object({
    name: z.string(),
    parents: z.array(z.string()).optional(),
    timeout: workflow_1.HatchetTimeoutSchema.optional(),
    retries: z.number().optional(),
    rate_limits: z.array(exports.CreateRateLimitSchema).optional(),
});
class ChildWorkflowRef {
    constructor(workflowRunId, parentWorkflowRunId, client) {
        this.workflowRunId = workflowRunId;
        this.parentWorkflowRunId = parentWorkflowRunId;
        this.client = client;
    }
    stream() {
        return __awaiter(this, void 0, void 0, function* () {
            const workflowRunId = yield this.workflowRunId;
            const listener = yield this.client.listener.getChildListener(workflowRunId, this.parentWorkflowRunId);
            return listener.stream();
        });
    }
    result() {
        return __awaiter(this, void 0, void 0, function* () {
            const listener = yield this.stream();
            return new Promise((resolve, reject) => {
                (() => __awaiter(this, void 0, void 0, function* () {
                    var _a, e_1, _b, _c;
                    try {
                        for (var _d = true, _e = __asyncValues(yield listener), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                            _c = _f.value;
                            _d = false;
                            const event = _c;
                            if (event.eventType === dispatcher_1.WorkflowRunEventType.WORKFLOW_RUN_EVENT_TYPE_FINISHED) {
                                if (event.results.some((r) => !!r.error)) {
                                    reject(event.results);
                                    return;
                                }
                                const result = event.results.reduce((acc, r) => (Object.assign(Object.assign({}, acc), { [r.stepReadableId]: JSON.parse(r.output || '{}') })), {});
                                resolve(result);
                                return;
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                }))();
            });
        });
    }
    toJSON() {
        return __awaiter(this, void 0, void 0, function* () {
            return JSON.stringify({
                workflowRunId: yield this.workflowRunId,
            });
        });
    }
}
class Context {
    constructor(action, client) {
        this.controller = new AbortController();
        this.overridesData = {};
        this.spawnIndex = 0;
        try {
            const data = (0, parse_1.parseJSON)(action.actionPayload);
            this.data = data;
            this.action = action;
            this.client = client;
            this.logger = new logger_1.Logger(`Context Logger`, client.config.log_level);
            // if this is a getGroupKeyRunId, the data is the workflow input
            if (action.getGroupKeyRunId !== '') {
                this.input = data;
            }
            else {
                this.input = data.input;
            }
            this.overridesData = data.overrides || {};
        }
        catch (e) {
            throw new hatchet_error_1.default(`Could not parse payload: ${e.message}`);
        }
    }
    stepOutput(step) {
        if (!this.data.parents) {
            throw new hatchet_error_1.default('Step output not found');
        }
        if (!this.data.parents[step]) {
            throw new hatchet_error_1.default(`Step output for '${step}' not found`);
        }
        return this.data.parents[step];
    }
    triggeredByEvent() {
        var _a;
        return ((_a = this.data) === null || _a === void 0 ? void 0 : _a.triggered_by) === 'event';
    }
    workflowInput() {
        return this.input;
    }
    workflowName() {
        return this.action.jobName;
    }
    userData() {
        var _a;
        return (_a = this.data) === null || _a === void 0 ? void 0 : _a.user_data;
    }
    stepName() {
        return this.action.stepName;
    }
    workflowRunId() {
        return this.action.workflowRunId;
    }
    playground(name, defaultValue = '') {
        if (name in this.overridesData) {
            return this.overridesData[name];
        }
        this.client.dispatcher.putOverridesData({
            stepRunId: this.action.stepRunId,
            path: name,
            value: JSON.stringify(defaultValue),
        });
        return defaultValue;
    }
    log(message, level) {
        const { stepRunId } = this.action;
        if (!stepRunId) {
            // log a warning
            this.logger.warn('cannot log from context without stepRunId');
            return;
        }
        this.client.event.putLog(stepRunId, message, level);
    }
    /**
     * Refreshes the timeout for the current step.
     * @param incrementBy - The interval by which to increment the timeout.
     *                     The interval should be specified in the format of '10s' for 10 seconds,
     *                     '1m' for 1 minute, or '1d' for 1 day.
     */
    refreshTimeout(incrementBy) {
        return __awaiter(this, void 0, void 0, function* () {
            const { stepRunId } = this.action;
            if (!stepRunId) {
                // log a warning
                this.logger.warn('cannot refresh timeout from context without stepRunId');
                return;
            }
            yield this.client.dispatcher.refreshTimeout(incrementBy, stepRunId);
        });
    }
    releaseSlot() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.dispatcher.client.releaseSlot({
                stepRunId: this.action.stepRunId,
            });
        });
    }
    putStream(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const { stepRunId } = this.action;
            if (!stepRunId) {
                // log a warning
                this.logger.warn('cannot log from context without stepRunId');
                return;
            }
            yield this.client.event.putStream(stepRunId, data);
        });
    }
    spawnWorkflow(workflowName, input, key) {
        const { workflowRunId, stepRunId } = this.action;
        const name = this.client.config.namespace + workflowName;
        const childWorkflowRunIdPromise = this.client.admin.run_workflow(name, input, {
            parentId: workflowRunId,
            parentStepRunId: stepRunId,
            childKey: key,
            childIndex: this.spawnIndex,
        });
        this.spawnIndex += 1;
        return new ChildWorkflowRef(childWorkflowRunIdPromise, workflowRunId, this.client);
    }
}
exports.Context = Context;
