"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
// eslint-disable-next-line no-shadow
var LogLevelEnum;
(function (LogLevelEnum) {
    LogLevelEnum[LogLevelEnum["OFF"] = -1] = "OFF";
    LogLevelEnum[LogLevelEnum["DEBUG"] = 0] = "DEBUG";
    LogLevelEnum[LogLevelEnum["INFO"] = 1] = "INFO";
    LogLevelEnum[LogLevelEnum["WARN"] = 2] = "WARN";
    LogLevelEnum[LogLevelEnum["ERROR"] = 3] = "ERROR";
})(LogLevelEnum || (LogLevelEnum = {}));
class Logger {
    constructor(context, logLevel = 'INFO') {
        this.logLevel = logLevel;
        this.context = context;
    }
    log(level, message, color) {
        if (LogLevelEnum[level] >= LogLevelEnum[this.logLevel]) {
            const time = new Date().toLocaleString('en-US', {
                month: '2-digit',
                day: '2-digit',
                year: '2-digit',
                hour: '2-digit',
                minute: '2-digit',
                second: '2-digit',
            });
            // eslint-disable-next-line no-console
            console.log(`🪓 ${process.pid} | ${time} ${color && `\x1b[${color}m`} [${level}/${this.context}] ${message}\x1b[0m`);
        }
    }
    debug(message) {
        this.log('DEBUG', message, '35');
    }
    info(message) {
        this.log('INFO', message);
    }
    green(message) {
        this.log('INFO', message, '32');
    }
    warn(message) {
        this.log('WARN', message, '93');
    }
    error(message) {
        this.log('ERROR', message, '91');
    }
}
exports.Logger = Logger;
exports.default = Logger;
