import * as z from 'zod';
import { CreateStep } from './step';
import { ConcurrencyLimitStrategy as PbConcurrencyLimitStrategy } from './protoc/workflows';
declare const StepsSchema: z.ZodArray<z.ZodObject<{
    name: z.ZodString;
    parents: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
    timeout: z.ZodOptional<z.ZodString>;
    retries: z.ZodOptional<z.ZodNumber>;
    rate_limits: z.ZodOptional<z.ZodArray<z.ZodObject<{
        key: z.ZodString;
        units: z.ZodNumber;
    }, "strip", z.ZodTypeAny, {
        key: string;
        units: number;
    }, {
        key: string;
        units: number;
    }>, "many">>;
}, "strip", z.ZodTypeAny, {
    name: string;
    timeout?: string | undefined;
    parents?: string[] | undefined;
    retries?: number | undefined;
    rate_limits?: {
        key: string;
        units: number;
    }[] | undefined;
}, {
    name: string;
    timeout?: string | undefined;
    parents?: string[] | undefined;
    retries?: number | undefined;
    rate_limits?: {
        key: string;
        units: number;
    }[] | undefined;
}>, "many">;
export type Steps = z.infer<typeof StepsSchema>;
export declare const ConcurrencyLimitStrategy: typeof PbConcurrencyLimitStrategy;
export declare const WorkflowConcurrency: z.ZodObject<{
    name: z.ZodString;
    maxRuns: z.ZodOptional<z.ZodNumber>;
    limitStrategy: z.ZodOptional<z.ZodNativeEnum<typeof PbConcurrencyLimitStrategy>>;
}, "strip", z.ZodTypeAny, {
    name: string;
    maxRuns?: number | undefined;
    limitStrategy?: PbConcurrencyLimitStrategy | undefined;
}, {
    name: string;
    maxRuns?: number | undefined;
    limitStrategy?: PbConcurrencyLimitStrategy | undefined;
}>;
export declare const HatchetTimeoutSchema: z.ZodString;
export declare const CreateWorkflowSchema: z.ZodObject<{
    id: z.ZodString;
    description: z.ZodString;
    version: z.ZodOptional<z.ZodString>;
    scheduleTimeout: z.ZodOptional<z.ZodString>;
    /**
     * @deprecated Workflow timeout is deprecated. Use step timeouts instead.
     */
    timeout: z.ZodOptional<z.ZodString>;
    on: z.ZodUnion<[z.ZodObject<{
        cron: z.ZodString;
        event: z.ZodUndefined;
    }, "strip", z.ZodTypeAny, {
        cron: string;
        event?: undefined;
    }, {
        cron: string;
        event?: undefined;
    }>, z.ZodObject<{
        cron: z.ZodUndefined;
        event: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        event: string;
        cron?: undefined;
    }, {
        event: string;
        cron?: undefined;
    }>]>;
    steps: z.ZodArray<z.ZodObject<{
        name: z.ZodString;
        parents: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
        timeout: z.ZodOptional<z.ZodString>;
        retries: z.ZodOptional<z.ZodNumber>;
        rate_limits: z.ZodOptional<z.ZodArray<z.ZodObject<{
            key: z.ZodString;
            units: z.ZodNumber;
        }, "strip", z.ZodTypeAny, {
            key: string;
            units: number;
        }, {
            key: string;
            units: number;
        }>, "many">>;
    }, "strip", z.ZodTypeAny, {
        name: string;
        timeout?: string | undefined;
        parents?: string[] | undefined;
        retries?: number | undefined;
        rate_limits?: {
            key: string;
            units: number;
        }[] | undefined;
    }, {
        name: string;
        timeout?: string | undefined;
        parents?: string[] | undefined;
        retries?: number | undefined;
        rate_limits?: {
            key: string;
            units: number;
        }[] | undefined;
    }>, "many">;
    onFailure: z.ZodOptional<z.ZodObject<{
        name: z.ZodString;
        parents: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
        timeout: z.ZodOptional<z.ZodString>;
        retries: z.ZodOptional<z.ZodNumber>;
        rate_limits: z.ZodOptional<z.ZodArray<z.ZodObject<{
            key: z.ZodString;
            units: z.ZodNumber;
        }, "strip", z.ZodTypeAny, {
            key: string;
            units: number;
        }, {
            key: string;
            units: number;
        }>, "many">>;
    }, "strip", z.ZodTypeAny, {
        name: string;
        timeout?: string | undefined;
        parents?: string[] | undefined;
        retries?: number | undefined;
        rate_limits?: {
            key: string;
            units: number;
        }[] | undefined;
    }, {
        name: string;
        timeout?: string | undefined;
        parents?: string[] | undefined;
        retries?: number | undefined;
        rate_limits?: {
            key: string;
            units: number;
        }[] | undefined;
    }>>;
}, "strip", z.ZodTypeAny, {
    description: string;
    steps: {
        name: string;
        timeout?: string | undefined;
        parents?: string[] | undefined;
        retries?: number | undefined;
        rate_limits?: {
            key: string;
            units: number;
        }[] | undefined;
    }[];
    id: string;
    on: {
        cron: string;
        event?: undefined;
    } | {
        event: string;
        cron?: undefined;
    };
    version?: string | undefined;
    scheduleTimeout?: string | undefined;
    timeout?: string | undefined;
    onFailure?: {
        name: string;
        timeout?: string | undefined;
        parents?: string[] | undefined;
        retries?: number | undefined;
        rate_limits?: {
            key: string;
            units: number;
        }[] | undefined;
    } | undefined;
}, {
    description: string;
    steps: {
        name: string;
        timeout?: string | undefined;
        parents?: string[] | undefined;
        retries?: number | undefined;
        rate_limits?: {
            key: string;
            units: number;
        }[] | undefined;
    }[];
    id: string;
    on: {
        cron: string;
        event?: undefined;
    } | {
        event: string;
        cron?: undefined;
    };
    version?: string | undefined;
    scheduleTimeout?: string | undefined;
    timeout?: string | undefined;
    onFailure?: {
        name: string;
        timeout?: string | undefined;
        parents?: string[] | undefined;
        retries?: number | undefined;
        rate_limits?: {
            key: string;
            units: number;
        }[] | undefined;
    } | undefined;
}>;
export interface Workflow extends z.infer<typeof CreateWorkflowSchema> {
    concurrency?: z.infer<typeof WorkflowConcurrency> & {
        key: (ctx: any) => string;
    };
    steps: CreateStep<any, any>[];
    onFailure?: CreateStep<any, any>;
}
export {};
